@@ -1,5 +1,5 @@
 /* api.h
- *  Copyright (C) 2001-2010, Parrot Foundation.
+ *  Copyright (C) 2001-2012, Parrot Foundation.
  *  Overview:
  *     Parrot's external embedding API. This functionality is only to be used
  *     by embedding applications, and only this functionality may be used by
@@ -17,6 +17,16 @@
 
 typedef Parrot_PMC (*imcc_hack_func_t)(Parrot_PMC, Parrot_String, int, const char **);
 
+PARROT_EXPORT
+PARROT_WARN_UNUSED_RESULT
+PARROT_CONST_FUNCTION
+const unsigned char * get_program_code(void);
+
+PARROT_EXPORT
+PARROT_WARN_UNUSED_RESULT
+PARROT_CONST_FUNCTION
+size_t get_program_code_size(void);
+
 /* Forward declaration of Parrot_confess. We can't include exceptions.h yet */
 PARROT_EXPORT
 PARROT_DOES_NOT_RETURN
@@ -54,10 +64,16 @@ Parrot_confess(ARGIN(const char *cond), ARGIN(const char *file), unsigned int li
 
 #endif /* NDEBUG */
 
+/* Static assertions are checked at compile type */
+#define PARROT_STATIC_ASSERT_HELPER(COND, MSG)  typedef char static_assertion_##MSG[(!!(COND))*2-1]
+#define PARROT_STATIC_ASSERT3(X, L)             PARROT_STATIC_ASSERT_HELPER(X, at_line_##L)
+#define PARROT_STATIC_ASSERT2(X, L)             PARROT_STATIC_ASSERT3(X, L)
+#define PARROT_STATIC_ASSERT(X)                 PARROT_STATIC_ASSERT2((X), __LINE__)
 
 typedef struct _Parrot_Init_Args {
     void *stacktop;
     const char *gc_system;
+    Parrot_Float4 gc_nursery_size;
     Parrot_Int gc_dynamic_threshold;
     Parrot_Int gc_min_threshold;
     Parrot_UInt hash_seed;
@@ -99,13 +115,6 @@ Parrot_Int Parrot_api_debug_flag(
 PARROT_API
 Parrot_Int Parrot_api_destroy_interpreter(Parrot_PMC interp_pmc);
 
-PARROT_API
-Parrot_Int Parrot_api_disassemble_bytecode(
-    Parrot_PMC interp_pmc,
-    Parrot_PMC pbc,
-    ARGIN_NULLOK(const char * const outfile),
-    Parrot_Int opts);
-
 PARROT_API
 Parrot_Int Parrot_api_flag(
     Parrot_PMC interp_pmc,
@@ -115,8 +124,9 @@ Parrot_Int Parrot_api_flag(
 PARROT_API
 Parrot_Int Parrot_api_get_compiler(
     Parrot_PMC interp_pmc,
-    Parrot_String type,
+    ARGIN(Parrot_String type),
     ARGOUT(Parrot_PMC *compiler))
+        __attribute__nonnull__(2)
         __attribute__nonnull__(3)
         FUNC_MODIFIES(*compiler);
 
@@ -124,25 +134,25 @@ PARROT_API
 Parrot_Int Parrot_api_get_exception_backtrace(
     Parrot_PMC interp_pmc,
     Parrot_PMC exception,
-    ARGOUT(Parrot_String * bt))
+    ARGOUT(Parrot_String *bt))
         __attribute__nonnull__(3)
-        FUNC_MODIFIES(* bt);
+        FUNC_MODIFIES(*bt);
 
 PARROT_API
 Parrot_Int Parrot_api_get_result(
     Parrot_PMC interp_pmc,
     ARGOUT(Parrot_Int *is_error),
-    ARGOUT(Parrot_PMC * exception),
+    ARGOUT(Parrot_PMC *exception),
     ARGOUT(Parrot_Int *exit_code),
-    ARGOUT(Parrot_String * errmsg))
+    ARGOUT(Parrot_String *errmsg))
         __attribute__nonnull__(2)
         __attribute__nonnull__(3)
         __attribute__nonnull__(4)
         __attribute__nonnull__(5)
         FUNC_MODIFIES(*is_error)
-        FUNC_MODIFIES(* exception)
+        FUNC_MODIFIES(*exception)
         FUNC_MODIFIES(*exit_code)
-        FUNC_MODIFIES(* errmsg);
+        FUNC_MODIFIES(*errmsg);
 
 PARROT_API
 Parrot_Int Parrot_api_get_runtime_path(
@@ -151,29 +161,11 @@ Parrot_Int Parrot_api_get_runtime_path(
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*runtime);
 
-PARROT_API
-Parrot_Int Parrot_api_load_bytecode_bytes(
-    Parrot_PMC interp_pmc,
-    ARGIN(const unsigned char * const pbc),
-    Parrot_Int bytecode_size,
-    ARGOUT(Parrot_PMC * pbcpmc))
-        __attribute__nonnull__(2)
-        __attribute__nonnull__(4)
-        FUNC_MODIFIES(* pbcpmc);
-
-PARROT_API
-Parrot_Int Parrot_api_load_bytecode_file(
-    Parrot_PMC interp_pmc,
-    ARGIN(Parrot_String filename),
-    ARGOUT(Parrot_PMC * pbc))
-        __attribute__nonnull__(2)
-        __attribute__nonnull__(3)
-        FUNC_MODIFIES(* pbc);
-
 PARROT_API
 Parrot_Int Parrot_api_load_language(
     Parrot_PMC interp_pmc,
-    Parrot_String lang);
+    ARGIN(Parrot_String lang))
+        __attribute__nonnull__(2);
 
 PARROT_API
 Parrot_Int Parrot_api_make_interpreter(
@@ -184,38 +176,20 @@ Parrot_Int Parrot_api_make_interpreter(
         __attribute__nonnull__(4)
         FUNC_MODIFIES(*interp);
 
-PARROT_API
-Parrot_Int Parrot_api_ready_bytecode(
-    Parrot_PMC interp_pmc,
-    Parrot_PMC pbc,
-    ARGOUT(Parrot_PMC *main_sub))
-        __attribute__nonnull__(3)
-        FUNC_MODIFIES(*main_sub);
-
 PARROT_API
 Parrot_Int Parrot_api_reset_call_signature(
     Parrot_PMC interp_pmc,
-    Parrot_PMC ctx);
-
-PARROT_API
-Parrot_Int Parrot_api_run_bytecode(
-    Parrot_PMC interp_pmc,
-    Parrot_PMC pbc,
-    Parrot_PMC mainargs);
-
-PARROT_API
-Parrot_Int Parrot_api_serialize_bytecode_pmc(
-    Parrot_PMC interp_pmc,
-    Parrot_PMC pbc,
-    ARGOUT(Parrot_String * bc))
-        __attribute__nonnull__(3)
-        FUNC_MODIFIES(* bc);
+    ARGMOD(Parrot_PMC ctx))
+        __attribute__nonnull__(2)
+        FUNC_MODIFIES(ctx);
 
 PARROT_API
 Parrot_Int Parrot_api_set_compiler(
     Parrot_PMC interp_pmc,
-    Parrot_String type,
-    Parrot_PMC compiler);
+    ARGIN(Parrot_String type),
+    ARGIN(Parrot_PMC compiler))
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3);
 
 PARROT_API
 Parrot_Int Parrot_api_set_configuration_hash(
@@ -225,7 +199,7 @@ Parrot_Int Parrot_api_set_configuration_hash(
 PARROT_API
 Parrot_Int Parrot_api_set_executable_name(
     Parrot_PMC interp_pmc,
-    ARGIN(const char * name))
+    ARGIN(const char *name))
         __attribute__nonnull__(2);
 
 PARROT_API
@@ -235,6 +209,15 @@ Parrot_Int Parrot_api_set_runcore(
     Parrot_UInt trace)
         __attribute__nonnull__(2);
 
+PARROT_API
+Parrot_Int Parrot_api_set_stdhandle(
+    Parrot_PMC interp_pmc,
+    Parrot_PMC handle,
+    Parrot_Int fileno,
+    ARGOUT(Parrot_PMC *old_handle))
+        __attribute__nonnull__(4)
+        FUNC_MODIFIES(*old_handle);
+
 PARROT_API
 Parrot_Int Parrot_api_set_warnings(Parrot_PMC interp_pmc, Parrot_Int flags);
 
@@ -261,12 +244,6 @@ Parrot_Int Parrot_api_wrap_pointer(
         __attribute__nonnull__(4)
         FUNC_MODIFIES(*pmc);
 
-PARROT_API
-Parrot_Int Parrot_api_write_bytecode_to_file(
-    Parrot_PMC interp_pmc,
-    Parrot_PMC pbc,
-    Parrot_String filename);
-
 #define ASSERT_ARGS_Parrot_api_add_dynext_search_path \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(path))
@@ -279,11 +256,10 @@ Parrot_Int Parrot_api_write_bytecode_to_file(
 #define ASSERT_ARGS_Parrot_api_debug_flag __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_api_destroy_interpreter \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
-#define ASSERT_ARGS_Parrot_api_disassemble_bytecode \
-     __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_api_flag __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_api_get_compiler __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(compiler))
+       PARROT_ASSERT_ARG(type) \
+    , PARROT_ASSERT_ARG(compiler))
 #define ASSERT_ARGS_Parrot_api_get_exception_backtrace \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(bt))
@@ -294,25 +270,16 @@ Parrot_Int Parrot_api_write_bytecode_to_file(
     , PARROT_ASSERT_ARG(errmsg))
 #define ASSERT_ARGS_Parrot_api_get_runtime_path __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(runtime))
-#define ASSERT_ARGS_Parrot_api_load_bytecode_bytes \
-     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(pbc) \
-    , PARROT_ASSERT_ARG(pbcpmc))
-#define ASSERT_ARGS_Parrot_api_load_bytecode_file __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(filename) \
-    , PARROT_ASSERT_ARG(pbc))
-#define ASSERT_ARGS_Parrot_api_load_language __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
+#define ASSERT_ARGS_Parrot_api_load_language __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(lang))
 #define ASSERT_ARGS_Parrot_api_make_interpreter __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp))
-#define ASSERT_ARGS_Parrot_api_ready_bytecode __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(main_sub))
 #define ASSERT_ARGS_Parrot_api_reset_call_signature \
-     __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
-#define ASSERT_ARGS_Parrot_api_run_bytecode __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
-#define ASSERT_ARGS_Parrot_api_serialize_bytecode_pmc \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(bc))
-#define ASSERT_ARGS_Parrot_api_set_compiler __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
+       PARROT_ASSERT_ARG(ctx))
+#define ASSERT_ARGS_Parrot_api_set_compiler __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(type) \
+    , PARROT_ASSERT_ARG(compiler))
 #define ASSERT_ARGS_Parrot_api_set_configuration_hash \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_api_set_executable_name \
@@ -320,6 +287,8 @@ Parrot_Int Parrot_api_write_bytecode_to_file(
        PARROT_ASSERT_ARG(name))
 #define ASSERT_ARGS_Parrot_api_set_runcore __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(corename))
+#define ASSERT_ARGS_Parrot_api_set_stdhandle __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(old_handle))
 #define ASSERT_ARGS_Parrot_api_set_warnings __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_api_toggle_gc __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_api_unwrap_pointer __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
@@ -327,10 +296,85 @@ Parrot_Int Parrot_api_write_bytecode_to_file(
     , PARROT_ASSERT_ARG(size))
 #define ASSERT_ARGS_Parrot_api_wrap_pointer __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(pmc))
+/* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
+/* HEADERIZER END: src/embed/api.c */
+
+/* HEADERIZER BEGIN: src/embed/bytecode.c */
+/* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
+
+PARROT_API
+Parrot_Int Parrot_api_disassemble_bytecode(
+    Parrot_PMC interp_pmc,
+    Parrot_PMC pbc,
+    ARGIN_NULLOK(const char * const outfile),
+    Parrot_Int opts);
+
+PARROT_API
+Parrot_Int Parrot_api_load_bytecode_bytes(
+    Parrot_PMC interp_pmc,
+    ARGIN(const unsigned char * const pbc),
+    Parrot_Int bytecode_size,
+    ARGOUT(Parrot_PMC * pbcpmc))
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(4)
+        FUNC_MODIFIES(* pbcpmc);
+
+PARROT_API
+Parrot_Int Parrot_api_load_bytecode_file(
+    Parrot_PMC interp_pmc,
+    ARGIN(Parrot_String filename),
+    ARGOUT(Parrot_PMC * pbc))
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(* pbc);
+
+PARROT_API
+Parrot_Int Parrot_api_ready_bytecode(
+    Parrot_PMC interp_pmc,
+    Parrot_PMC pbc,
+    ARGOUT(Parrot_PMC *main_sub))
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(*main_sub);
+
+PARROT_API
+Parrot_Int Parrot_api_run_bytecode(
+    Parrot_PMC interp_pmc,
+    Parrot_PMC pbc,
+    Parrot_PMC args);
+
+PARROT_API
+Parrot_Int Parrot_api_serialize_bytecode_pmc(
+    Parrot_PMC interp_pmc,
+    Parrot_PMC pbc,
+    ARGOUT(Parrot_String * bc))
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(* bc);
+
+PARROT_API
+Parrot_Int Parrot_api_write_bytecode_to_file(
+    Parrot_PMC interp_pmc,
+    Parrot_PMC pbc,
+    Parrot_String filename);
+
+#define ASSERT_ARGS_Parrot_api_disassemble_bytecode \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
+#define ASSERT_ARGS_Parrot_api_load_bytecode_bytes \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(pbc) \
+    , PARROT_ASSERT_ARG(pbcpmc))
+#define ASSERT_ARGS_Parrot_api_load_bytecode_file __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(filename) \
+    , PARROT_ASSERT_ARG(pbc))
+#define ASSERT_ARGS_Parrot_api_ready_bytecode __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(main_sub))
+#define ASSERT_ARGS_Parrot_api_run_bytecode __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
+#define ASSERT_ARGS_Parrot_api_serialize_bytecode_pmc \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(bc))
 #define ASSERT_ARGS_Parrot_api_write_bytecode_to_file \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
-/* HEADERIZER END: src/embed/api.c */
+/* HEADERIZER END: src/embed/bytecode.c */
 
 /* HEADERIZER BEGIN: src/embed/strings.c */
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
@@ -466,6 +510,14 @@ Parrot_Int Parrot_api_string_import_wchar(
 /* HEADERIZER BEGIN: src/embed/pmc.c */
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 
+PARROT_API
+Parrot_Int Parrot_api_pmc_box_float(
+    Parrot_PMC interp_pmc,
+    Parrot_Float value,
+    ARGOUT(Parrot_PMC * float_pmc))
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(* float_pmc);
+
 PARROT_API
 Parrot_Int Parrot_api_pmc_box_integer(
     Parrot_PMC interp_pmc,
@@ -601,6 +653,13 @@ Parrot_Int Parrot_api_pmc_keep_alive(
     Parrot_PMC pmc,
     Parrot_Int alive);
 
+PARROT_API
+Parrot_Int Parrot_api_pmc_new_call_object(
+    Parrot_PMC interp_pmc,
+    ARGOUT(Parrot_PMC *cc))
+        __attribute__nonnull__(2)
+        FUNC_MODIFIES(*cc);
+
 PARROT_API
 Parrot_Int Parrot_api_pmc_new_from_class(
     ARGIN(Parrot_PMC interp_pmc),
@@ -615,7 +674,7 @@ Parrot_Int Parrot_api_pmc_new_from_class(
 PARROT_API
 Parrot_Int Parrot_api_pmc_null(
     ARGIN(Parrot_PMC interp_pmc),
-    ARGMOD(Parrot_PMC *pmctonull))
+    ARGOUT(Parrot_PMC *pmctonull))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*pmctonull);
@@ -687,6 +746,14 @@ Parrot_Int Parrot_api_pmc_set_string(
         __attribute__nonnull__(2)
         __attribute__nonnull__(3);
 
+PARROT_API
+Parrot_Int Parrot_api_pmc_setup_signature(
+    Parrot_PMC interp_pmc,
+    Parrot_PMC callcontext,
+    ARGIN(const char * const signature),
+    ...)
+        __attribute__nonnull__(3);
+
 PARROT_API
 Parrot_Int Parrot_api_pmc_wrap_string_array(
     ARGIN(Parrot_PMC interp_pmc),
@@ -698,6 +765,8 @@ Parrot_Int Parrot_api_pmc_wrap_string_array(
         __attribute__nonnull__(4)
         FUNC_MODIFIES(* args);
 
+#define ASSERT_ARGS_Parrot_api_pmc_box_float __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(float_pmc))
 #define ASSERT_ARGS_Parrot_api_pmc_box_integer __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(int_pmc))
 #define ASSERT_ARGS_Parrot_api_pmc_box_string __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
@@ -748,6 +817,9 @@ Parrot_Int Parrot_api_pmc_wrap_string_array(
     , PARROT_ASSERT_ARG(sub) \
     , PARROT_ASSERT_ARG(signature))
 #define ASSERT_ARGS_Parrot_api_pmc_keep_alive __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
+#define ASSERT_ARGS_Parrot_api_pmc_new_call_object \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(cc))
 #define ASSERT_ARGS_Parrot_api_pmc_new_from_class __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp_pmc) \
     , PARROT_ASSERT_ARG(class_pmc) \
@@ -779,6 +851,9 @@ Parrot_Int Parrot_api_pmc_wrap_string_array(
        PARROT_ASSERT_ARG(interp_pmc) \
     , PARROT_ASSERT_ARG(pmc) \
     , PARROT_ASSERT_ARG(value))
+#define ASSERT_ARGS_Parrot_api_pmc_setup_signature \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(signature))
 #define ASSERT_ARGS_Parrot_api_pmc_wrap_string_array \
      __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp_pmc) \
