@@ -109,8 +109,9 @@ void Parrot_IOData_mark(PARROT_INTERP, ARGIN(ParrotIOData *piodata))
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 
 PARROT_EXPORT
-INTVAL Parrot_io_close_handle(PARROT_INTERP, ARGMOD_NULLOK(PMC *pmc))
+INTVAL Parrot_io_close_handle(PARROT_INTERP, ARGMOD(PMC *pmc))
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         FUNC_MODIFIES(*pmc);
 
 PARROT_EXPORT
@@ -133,25 +134,28 @@ PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
 PMC * Parrot_io_fdopen(PARROT_INTERP,
-    ARGIN_NULLOK(PMC *pmc),
+    ARGIN(PMC *pmc),
     PIOHANDLE fd,
     ARGIN(STRING *sflags))
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         __attribute__nonnull__(4);
 
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
 PMC * Parrot_io_fdopen_flags(PARROT_INTERP,
-    ARGMOD_NULLOK(PMC *filehandle),
+    ARGMOD(PMC *filehandle),
     PIOHANDLE fd,
     INTVAL flags)
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         FUNC_MODIFIES(*filehandle);
 
 PARROT_EXPORT
-void Parrot_io_flush_handle(PARROT_INTERP, ARGMOD_NULLOK(PMC *pmc))
+void Parrot_io_flush_handle(PARROT_INTERP, ARGMOD(PMC *pmc))
         __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
         FUNC_MODIFIES(*pmc);
 
 PARROT_EXPORT
@@ -172,6 +176,13 @@ PIOHANDLE Parrot_io_getfd(PARROT_INTERP, ARGIN(const PMC *pmc))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2);
 
+PARROT_EXPORT
+PARROT_WARN_UNUSED_RESULT
+INTVAL Parrot_io_is_async(PARROT_INTERP, ARGMOD(PMC *pmc))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        FUNC_MODIFIES(*pmc);
+
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 INTVAL Parrot_io_is_closed(PARROT_INTERP, ARGIN(PMC *pmc))
@@ -196,16 +207,20 @@ PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
 PMC * Parrot_io_open_handle(PARROT_INTERP,
-    ARGIN_NULLOK(PMC *pmc),
-    ARGIN_NULLOK(STRING *path),
-    ARGIN_NULLOK(STRING *mode))
-        __attribute__nonnull__(1);
+    ARGIN(PMC *pmc),
+    ARGIN(STRING *path),
+    ARGIN(STRING *mode))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        __attribute__nonnull__(4);
 
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 INTVAL Parrot_io_parse_open_flags(PARROT_INTERP,
-    ARGIN_NULLOK(const STRING *mode_str))
-        __attribute__nonnull__(1);
+    ARGIN(const STRING *mode_str))
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2);
 
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
@@ -235,11 +250,10 @@ Parrot_io_printf(PARROT_INTERP,
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
-INTVAL Parrot_io_putps(PARROT_INTERP,
-    ARGMOD(PMC *pmc),
-    ARGMOD_NULLOK(STRING *s))
+INTVAL Parrot_io_putps(PARROT_INTERP, ARGMOD(PMC *pmc), ARGMOD(STRING *s))
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
         FUNC_MODIFIES(*pmc)
         FUNC_MODIFIES(*s);
 
@@ -250,6 +264,17 @@ INTVAL Parrot_io_puts(PARROT_INTERP, ARGMOD(PMC *pmc), ARGIN(const char *s))
         __attribute__nonnull__(3)
         FUNC_MODIFIES(*pmc);
 
+PARROT_EXPORT
+PARROT_CANNOT_RETURN_NULL
+PMC * Parrot_io_read_byte_buffer_pmc(PARROT_INTERP,
+    ARGMOD(PMC *handle),
+    ARGMOD_NULLOK(PMC *buffer),
+    INTVAL length)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        FUNC_MODIFIES(*handle)
+        FUNC_MODIFIES(*buffer);
+
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CANNOT_RETURN_NULL
@@ -326,8 +351,19 @@ PIOOFF_T Parrot_io_make_offset_pmc(PARROT_INTERP, ARGMOD(PMC *pmc))
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*pmc);
 
+INTVAL Parrot_io_write_byte_buffer_pmc(PARROT_INTERP,
+    ARGMOD(PMC * handle),
+    ARGMOD(PMC *buffer),
+    INTVAL length)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(* handle)
+        FUNC_MODIFIES(*buffer);
+
 #define ASSERT_ARGS_Parrot_io_close_handle __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pmc))
 #define ASSERT_ARGS_Parrot_io_eof __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc))
@@ -335,11 +371,14 @@ PIOOFF_T Parrot_io_make_offset_pmc(PARROT_INTERP, ARGMOD(PMC *pmc))
        PARROT_ASSERT_ARG(s))
 #define ASSERT_ARGS_Parrot_io_fdopen __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pmc) \
     , PARROT_ASSERT_ARG(sflags))
 #define ASSERT_ARGS_Parrot_io_fdopen_flags __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(filehandle))
 #define ASSERT_ARGS_Parrot_io_flush_handle __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pmc))
 #define ASSERT_ARGS_Parrot_io_fprintf __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc) \
@@ -347,6 +386,9 @@ PIOOFF_T Parrot_io_make_offset_pmc(PARROT_INTERP, ARGMOD(PMC *pmc))
 #define ASSERT_ARGS_Parrot_io_getfd __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc))
+#define ASSERT_ARGS_Parrot_io_is_async __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pmc))
 #define ASSERT_ARGS_Parrot_io_is_closed __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc))
@@ -356,9 +398,13 @@ PIOOFF_T Parrot_io_make_offset_pmc(PARROT_INTERP, ARGMOD(PMC *pmc))
 #define ASSERT_ARGS_Parrot_io_make_offset __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_io_make_offset32 __attribute__unused__ int _ASSERT_ARGS_CHECK = (0)
 #define ASSERT_ARGS_Parrot_io_open_handle __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(pmc) \
+    , PARROT_ASSERT_ARG(path) \
+    , PARROT_ASSERT_ARG(mode))
 #define ASSERT_ARGS_Parrot_io_parse_open_flags __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
-       PARROT_ASSERT_ARG(interp))
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(mode_str))
 #define ASSERT_ARGS_Parrot_io_peek __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc))
@@ -370,11 +416,16 @@ PIOOFF_T Parrot_io_make_offset_pmc(PARROT_INTERP, ARGMOD(PMC *pmc))
     , PARROT_ASSERT_ARG(s))
 #define ASSERT_ARGS_Parrot_io_putps __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
-    , PARROT_ASSERT_ARG(pmc))
+    , PARROT_ASSERT_ARG(pmc) \
+    , PARROT_ASSERT_ARG(s))
 #define ASSERT_ARGS_Parrot_io_puts __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc) \
     , PARROT_ASSERT_ARG(s))
+#define ASSERT_ARGS_Parrot_io_read_byte_buffer_pmc \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(handle))
 #define ASSERT_ARGS_Parrot_io_readline __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc))
@@ -402,6 +453,11 @@ PIOOFF_T Parrot_io_make_offset_pmc(PARROT_INTERP, ARGMOD(PMC *pmc))
 #define ASSERT_ARGS_Parrot_io_make_offset_pmc __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc))
+#define ASSERT_ARGS_Parrot_io_write_byte_buffer_pmc \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(handle) \
+    , PARROT_ASSERT_ARG(buffer))
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 /* HEADERIZER END: src/io/api.c */
 
@@ -430,7 +486,7 @@ INTVAL Parrot_io_peek_buffer(PARROT_INTERP, ARGMOD(PMC *filehandle))
 PARROT_WARN_UNUSED_RESULT
 size_t Parrot_io_read_buffer(PARROT_INTERP,
     ARGMOD(PMC *filehandle),
-    ARGMOD(char *dest),
+    ARGOUT(char *dest),
     size_t len)
         __attribute__nonnull__(1)
         __attribute__nonnull__(2)
@@ -530,36 +586,37 @@ void Parrot_io_flush_filehandle(PARROT_INTERP, ARGMOD(PMC *pmc))
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
-unsigned char * Parrot_io_get_buffer_end(SHIM_INTERP,
+unsigned char * Parrot_io_get_buffer_end(PARROT_INTERP,
     ARGIN(const PMC *filehandle))
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
 PARROT_CAN_RETURN_NULL
-unsigned char * Parrot_io_get_buffer_next(SHIM_INTERP,
+unsigned char * Parrot_io_get_buffer_next(PARROT_INTERP,
     ARGIN(const PMC *filehandle))
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
 PARROT_CAN_RETURN_NULL
-unsigned char * Parrot_io_get_buffer_start(SHIM_INTERP,
+unsigned char * Parrot_io_get_buffer_start(PARROT_INTERP,
     ARGIN(PMC *filehandle))
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
-PIOOFF_T Parrot_io_get_file_position(SHIM_INTERP,
+PIOOFF_T Parrot_io_get_file_position(PARROT_INTERP,
     ARGIN(const PMC *filehandle))
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
-INTVAL Parrot_io_get_flags(SHIM_INTERP, ARGIN(const PMC *filehandle))
+INTVAL Parrot_io_get_flags(PARROT_INTERP, ARGIN(const PMC *filehandle))
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
 PARROT_WARN_UNUSED_RESULT
-PIOHANDLE Parrot_io_get_os_handle(SHIM_INTERP, ARGIN(const PMC *filehandle))
+PIOHANDLE Parrot_io_get_os_handle(PARROT_INTERP,
+    ARGIN(const PMC *filehandle))
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
@@ -569,18 +626,20 @@ INTVAL Parrot_io_is_closed_filehandle(PARROT_INTERP, ARGIN(const PMC *pmc))
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
-void Parrot_io_set_file_position(SHIM_INTERP,
+void Parrot_io_set_file_position(PARROT_INTERP,
     ARGMOD(PMC *filehandle),
     PIOOFF_T file_pos)
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*filehandle);
 
 PARROT_EXPORT
-void Parrot_io_set_flags(SHIM_INTERP, ARGIN(PMC *filehandle), INTVAL flags)
+void Parrot_io_set_flags(PARROT_INTERP,
+    ARGIN(PMC *filehandle),
+    INTVAL flags)
         __attribute__nonnull__(2);
 
 PARROT_EXPORT
-void Parrot_io_set_os_handle(SHIM_INTERP,
+void Parrot_io_set_os_handle(PARROT_INTERP,
     ARGMOD(PMC *filehandle),
     PIOHANDLE file_descriptor)
         __attribute__nonnull__(2)
@@ -593,12 +652,14 @@ void Parrot_io_clear_buffer(PARROT_INTERP, ARGMOD(PMC *filehandle))
         FUNC_MODIFIES(*filehandle);
 
 PARROT_CAN_RETURN_NULL
-INTVAL Parrot_io_get_buffer_flags(SHIM_INTERP, ARGIN(const PMC *filehandle))
+INTVAL Parrot_io_get_buffer_flags(PARROT_INTERP,
+    ARGIN(const PMC *filehandle))
         __attribute__nonnull__(2);
 
 PARROT_CAN_RETURN_NULL
 PARROT_WARN_UNUSED_RESULT
-size_t Parrot_io_get_buffer_size(SHIM_INTERP, ARGIN(const PMC *filehandle))
+size_t Parrot_io_get_buffer_size(PARROT_INTERP,
+    ARGIN(const PMC *filehandle))
         __attribute__nonnull__(2);
 
 PARROT_WARN_UNUSED_RESULT
@@ -610,31 +671,31 @@ STRING * Parrot_io_make_string(PARROT_INTERP,
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*buf);
 
-void Parrot_io_set_buffer_end(SHIM_INTERP,
+void Parrot_io_set_buffer_end(PARROT_INTERP,
     ARGMOD(PMC *filehandle),
     ARGIN_NULLOK(unsigned char *new_end))
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*filehandle);
 
-void Parrot_io_set_buffer_flags(SHIM_INTERP,
+void Parrot_io_set_buffer_flags(PARROT_INTERP,
     ARGMOD(PMC *filehandle),
     INTVAL new_flags)
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*filehandle);
 
-void Parrot_io_set_buffer_next(SHIM_INTERP,
+void Parrot_io_set_buffer_next(PARROT_INTERP,
     ARGMOD(PMC *filehandle),
     ARGIN_NULLOK(unsigned char *new_next))
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*filehandle);
 
-void Parrot_io_set_buffer_size(SHIM_INTERP,
+void Parrot_io_set_buffer_size(PARROT_INTERP,
     ARGMOD(PMC *filehandle),
     size_t new_size)
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*filehandle);
 
-void Parrot_io_set_buffer_start(SHIM_INTERP,
+void Parrot_io_set_buffer_start(PARROT_INTERP,
     ARGMOD(PMC *filehandle),
     ARGIN_NULLOK(unsigned char *new_start))
         __attribute__nonnull__(2)
@@ -776,7 +837,7 @@ INTVAL Parrot_io_socket_handle(PARROT_INTERP,
         FUNC_MODIFIES(*socket);
 
 PARROT_EXPORT
-void Parrot_io_socket_initialize_handle(SHIM_INTERP, ARGMOD(PMC *socket))
+void Parrot_io_socket_initialize_handle(PARROT_INTERP, ARGMOD(PMC *socket))
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*socket);
 
@@ -788,6 +849,25 @@ INTVAL Parrot_io_socket_is_closed(PARROT_INTERP, ARGMOD(PMC *socket))
         __attribute__nonnull__(2)
         FUNC_MODIFIES(*socket);
 
+INTVAL Parrot_io_socket_recv_to_buffer(PARROT_INTERP,
+    ARGMOD(PMC *socket),
+    ARGOUT(char * buffer),
+    size_t len)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(*socket)
+        FUNC_MODIFIES(* buffer);
+
+INTVAL Parrot_io_socket_send_from_buffer(PARROT_INTERP,
+    ARGMOD(PMC *socket),
+    ARGIN(const char *buffer),
+    size_t len)
+        __attribute__nonnull__(1)
+        __attribute__nonnull__(2)
+        __attribute__nonnull__(3)
+        FUNC_MODIFIES(*socket);
+
 #define ASSERT_ARGS_Parrot_io_accept_handle __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(pmc))
@@ -822,6 +902,16 @@ INTVAL Parrot_io_socket_is_closed(PARROT_INTERP, ARGMOD(PMC *socket))
 #define ASSERT_ARGS_Parrot_io_socket_is_closed __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
        PARROT_ASSERT_ARG(interp) \
     , PARROT_ASSERT_ARG(socket))
+#define ASSERT_ARGS_Parrot_io_socket_recv_to_buffer \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(socket) \
+    , PARROT_ASSERT_ARG(buffer))
+#define ASSERT_ARGS_Parrot_io_socket_send_from_buffer \
+     __attribute__unused__ int _ASSERT_ARGS_CHECK = (\
+       PARROT_ASSERT_ARG(interp) \
+    , PARROT_ASSERT_ARG(socket) \
+    , PARROT_ASSERT_ARG(buffer))
 /* Don't modify between HEADERIZER BEGIN / HEADERIZER END.  Your changes will be lost. */
 /* HEADERIZER END: src/io/socket_api.c */
 
