@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2010, Parrot Foundation.
+ * Copyright (C) 2003-2011, Parrot Foundation.
  */
 
 #ifndef PARROT_PLATFORM_INTERFACE_H_GUARD
@@ -78,6 +78,7 @@ PIOHANDLE Parrot_io_open(PARROT_INTERP, ARGIN(STRING *path), INTVAL flags);
 PIOHANDLE Parrot_io_dup(PARROT_INTERP, PIOHANDLE handle);
 INTVAL Parrot_io_close(PARROT_INTERP, PIOHANDLE handle);
 INTVAL Parrot_io_is_tty(PARROT_INTERP, PIOHANDLE fd);
+PARROT_CONST_FUNCTION
 INTVAL Parrot_io_getblksize(PIOHANDLE fd);
 INTVAL Parrot_io_flush(PARROT_INTERP, PIOHANDLE os_handle);
 size_t Parrot_io_read(PARROT_INTERP, PIOHANDLE os_handle, ARGOUT(char *buf), size_t len);
@@ -88,6 +89,9 @@ PIOHANDLE Parrot_io_open_pipe(PARROT_INTERP, ARGIN(STRING *command), INTVAL flag
         ARGOUT(INTVAL *pid_out));
 INTVAL Parrot_io_pipe(PARROT_INTERP, ARGMOD(PIOHANDLE *reader), ARGMOD(PIOHANDLE *writer));
 
+PARROT_EXPORT
+INTVAL Parrot_io_async(PARROT_INTERP, ARGMOD(PMC *pmc), INTVAL async);
+
 /*
  * Socket
  */
@@ -293,6 +297,7 @@ void Parrot_sleep(unsigned int seconds);
 
 PARROT_EXPORT
 void Parrot_usleep(unsigned int microseconds);
+void Parrot_floatval_sleep(FLOATVAL time);
 
 PARROT_EXPORT
 INTVAL Parrot_intval_time(void);
@@ -383,6 +388,7 @@ PARROT_EXPORT
 UHUGEINTVAL Parrot_hires_get_time(void);
 
 PARROT_EXPORT
+PARROT_CONST_FUNCTION
 UINTVAL     Parrot_hires_get_tick_duration(void);
 
 /*
@@ -406,6 +412,12 @@ size_t Parrot_sysmem_amount(Interp*);
 PARROT_EXPORT
 INTVAL Parrot_get_entropy(PARROT_INTERP);
 
+/*
+ * CPU Type
+ */
+PARROT_EXPORT
+STRING *Parrot_get_cpu_type(PARROT_INTERP);
+
 #endif /* PARROT_PLATFORM_INTERFACE_H_GUARD */
 
 /*
